library mkd_letter_bar;

import 'package:flutter/material.dart';
import 'common_define.dart';

class LetterBar extends StatefulWidget {
  // 定义一个回调函数（Function 是保留字，不是变量名）
  // 在 点击/按下拖动事件 调用，通知 FriendPage 选中的是哪个字母头
  final void Function(String str) letterBarCallBack;

  // 构造方法
  LetterBar({this.letterBarCallBack});

  @override
  _LetterBarState createState() => _LetterBarState();
}

// 通过点击LetterBar位置，获取字母下标
int getLetterIndex(BuildContext context, Offset globalPosition) {
  // 拿到 context 的box
  RenderBox box = context.findRenderObject();
  // 转 global坐标 → local坐标
  Offset localPosition = box.globalToLocal(globalPosition);
  // Y 值
  double y = localPosition.dy;
  // 整个 bar 的 height
  double barHeight = context.size.height;
  // 一个字母占用的 height
  double singleHeight = barHeight / 29;
  // 整除获取 letter下标，
  // 且给定一个范围，防止 _letter 数组越上下界
  int index = (y ~/ singleHeight).clamp(0, LETTERS.length - 1);
  // 返回 letter字符的 index值
  return index;
}

class _LetterBarState extends State<LetterBar> {
  // 右边 AB..Z# 的文字
  List<Widget> letterTexts;

  // 放大水滴的 Y值
  double _indicatorY = 0.0;

  // 放大水滴的 字母
  String _indicatorText = 'A';

  // 放大水滴 是否隐藏
  bool _indicatorHidden = true;

  // 初始化 letterTexts，在 build 调用一次
  void initLetterTexts() {
    // 每次 build 都会add，要先初始化
    letterTexts = [];
    for (String letterStr in LETTERS) {
      letterTexts.add(Expanded(
        child: Text(
          letterStr,
          style: TextStyle(
            fontSize: 11,
            fontWeight: FontWeight.w500,
            color: letterBarTextColor,
          ),
        ),
      ));
    }
  }

  @override
  Widget build(BuildContext context) {
    // 初始化 letterTexts
    initLetterTexts();

    return Positioned(
      top: ScreenHeight(context) * 0.1,
      right: 0,
      bottom: ScreenHeight(context) * 0.115,
      width: 110,
      child: Container(
          child: Row(
              mainAxisAlignment: MainAxisAlignment.spaceBetween,
              children: <Widget>[
                // 水滴放大UI
                Container(
                  width: 65,
                  // 整体水滴Stack 的位置，(0,0) 是中心点
                  alignment: Alignment(0, _indicatorY),
                  child: _indicatorHidden != true
                      ? Stack(
                    // 内部小部件的对齐。
                    // 这次的情况是水滴背景图充满了区域，调整文字的相对位置，(0,0) 是中心点
                    alignment: Alignment(-0.2, 0.02),
                    children: <Widget>[
                      // 水滴背景图
                      Image(
                        image: AssetImage('images/气泡.png',package: 'mkd_letter_bar'),
                        width: 80,
                        height: 80,
                        color: Color.fromRGBO(203, 203, 203, 1),
                      ),
                      // 水滴里的文字
                      Text(
                        _indicatorText,
                        style: TextStyle(
                          fontSize: 35,
                          fontWeight: FontWeight.w600,
                          color: Color.fromRGBO(254, 254, 254, 1),
                        ),
                      )
                    ],
                  )
                      : Container(),
                ),
                // 侧边的字母头
                Stack(
                  // 调整选中的字母头(白字绿底)的位置
                  alignment: Alignment(-0.198, _indicatorY * 0.872),
                  children: <Widget>[
                    // 侧边的字母头一列
                    GestureDetector(
                      // 点击手势
                      onVerticalDragDown: (DragDownDetails details) {
                        // 通过点击的位置，获取字母下标
                        int index = getLetterIndex(context, details.globalPosition);
                        // 使用在 stf 定义的回调函数，把选中的 字母头 传出去
                        widget.letterBarCallBack(LETTERS[index]);
                        // 显示水滴放大UI
                        setState(() {
                          _indicatorHidden = false;
                          _indicatorText = LETTERS[index];
                          _indicatorY = 2.28 / 28 * index - 1.15;
                        });
                      },
                      // 按下滑动手势
                      onVerticalDragUpdate: (DragUpdateDetails details) {
                        // 通过点击的位置，获取字母下标
                        int index = getLetterIndex(context, details.globalPosition);
                        // 使用在 stf 定义的回调函数，把选中的 字母头 传出去
                        widget.letterBarCallBack(LETTERS[index]);
                        // 显示水滴放大UI
                        setState(() {
                          _indicatorText = LETTERS[index];
                          _indicatorY = 2.28 / 28 * index - 1.15;
                        });
                      },
                      // 松开按下手势
                      onVerticalDragEnd: (DragEndDetails details) {
                        // 隐藏水滴放大UI
                        setState(() {
                          _indicatorHidden = true;
                        });
                      },
                      child: Container(
                        margin: EdgeInsets.only(right: 2),
                        width: 20,
                        child: Column(
                          children: letterTexts,
                        ),
                      ),
                    ),
                    // 选中的字母头，则直接创建显示一个白字绿底的，遮盖住原来的
                    Container(
                      // 显示/隐藏
                        child: _indicatorHidden != true
                            ? Container(
                          alignment: Alignment.center,
                          width: 14,
                          height: 14,
                          decoration: BoxDecoration(
                              color: Colors.green.shade400,
                              borderRadius: BorderRadius.circular(100.0)),
                          child: Text(
                            _indicatorText,
                            style: TextStyle(
                              fontSize: 11,
                              fontWeight: FontWeight.w500,
                              color: Colors.white,
                            ),
                          ),
                        )
                            : Container())
                  ],
                )
              ])),
    );
  }
}

// bar的字母
const LETTERS = [
  '🔍',
  '☆',
  'A',
  'B',
  'C',
  'D',
  'E',
  'F',
  'G',
  'H',
  'I',
  'J',
  'K',
  'L',
  'M',
  'N',
  'O',
  'P',
  'Q',
  'R',
  'S',
  'T',
  'U',
  'V',
  'W',
  'X',
  'Y',
  'Z',
  '#'
];
